import Drawnings.DirectionType;
import Drawnings.DrawningCleaningCar;
import Drawnings.DrawningTruck;
import MovementStrategy.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Random;
import java.util.List;
import java.util.LinkedList;

public class FormCleaningCar extends JFrame {
    protected DrawningTruck _drawningTruck;
    JPanel pictureBox;
    private JButton buttonCreateCleaningCar;
    private JButton buttonRight;
    private JButton buttonLeft;
    private JButton buttonDown;
    private JButton buttonUp;
    private JButton buttonCreateTruck;
    private JComboBox comboBoxStrategy;
    private JButton buttonStrategyStep;
    private List<JComponent> controls;

    private AbstractStrategy _strategy;

    public FormCleaningCar() {
        buttonUp.setName("buttonUp");
        buttonDown.setName("buttonDown");
        buttonLeft.setName("buttonLeft");
        buttonRight.setName("buttonRight");
        InitializeControlsRepaintList();

//        _strategy = null;
        buttonCreateCleaningCar.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                CreateObject("DrawningTruck");
            }
        });
        buttonCreateTruck.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                CreateObject("DrawningCleaningCar");
            }
        });
        ActionListener buttonMoveClickedListener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String buttonName = ((JButton) e.getSource()).getName();
                boolean result = false;

                switch (buttonName) {
                    case "buttonUp": {
                        result = _drawningTruck.MoveTransport(DirectionType.Up);
                    }
                    break;
                    case "buttonDown": {
                        result = _drawningTruck.MoveTransport(DirectionType.Down);
                    }
                    break;
                    case "buttonLeft": {
                        result = _drawningTruck.MoveTransport(DirectionType.Left);
                    }
                    break;
                    case "buttonRight": {
                        result = _drawningTruck.MoveTransport(DirectionType.Right);
                    }
                    break;

                }
                if (result)
                    Draw();

            }
        };
        buttonStrategyStep.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_drawningTruck == null) {
                    return;
                }
                if (comboBoxStrategy.isEnabled()) {
                    switch (comboBoxStrategy.getSelectedIndex()) {
                        case 0:
                            _strategy = new MoveToCenter();
                            break;
                        case 1:
                            _strategy = new MoveToBorder();
                            break;
                        default:
                            _strategy = null;
                            break;
                    }
                    if (_strategy == null) {
                        return;
                    }
                    _strategy.SetData(new MoveableTruck(_drawningTruck), pictureBox.getWidth(), pictureBox.getHeight());
                }
                if (_strategy == null) {
                    return;
                }
                comboBoxStrategy.setEnabled(false);
                _strategy.MakeStep();
                Draw();

                if (_strategy.GetStatus() == StrategyStatus.Finish) {
                    comboBoxStrategy.setEnabled(true);
                    _strategy = null;
                }
            }
        });
        buttonRight.addActionListener(buttonMoveClickedListener);
        buttonDown.addActionListener(buttonMoveClickedListener);
        buttonLeft.addActionListener(buttonMoveClickedListener);
        buttonUp.addActionListener(buttonMoveClickedListener);


        comboBoxStrategy.addItem("К центру");
        comboBoxStrategy.addItem("К краю");
    }
    private void CreateObject(String type) {
        Random random = new Random();
        switch (type) {
            case "DrawningTruck":
                _drawningTruck = new DrawningTruck(random.nextInt(100, 300), random.nextInt(100, 300),
                        new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)));
                break;
            case "DrawningCleaningCar":
                _drawningTruck = new DrawningCleaningCar(random.nextInt(100, 300), random.nextInt(100, 300),
                        new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)),
                        new Color(random.nextInt(256), random.nextInt(256), random.nextInt(256)),
                        random.nextBoolean(), random.nextBoolean(), random.nextBoolean());
                break;
            default: return;
        }
        _drawningTruck.SetPictureSize(pictureBox.getWidth(), pictureBox.getHeight());
        _drawningTruck.SetPosition(random.nextInt(10, 100), random.nextInt(0, 100));

        _strategy = null;
        comboBoxStrategy.setEnabled(true);
        Draw();
    }
    private void Draw() {
        if (_drawningTruck.getEntityTruck() == null ||
                pictureBox.getWidth() == 0 || pictureBox.getHeight() == 0)
            return;
        Graphics g = pictureBox.getGraphics();
        g.setColor(pictureBox.getBackground());
        g.fillRect(0,0, pictureBox.getWidth(), pictureBox.getHeight());
        _drawningTruck.DrawTransport(g);

        RepaintControls();

    }
    private void RepaintControls() {
        for (JComponent control : controls) {
            control.repaint();
        }
    }
    private void InitializeControlsRepaintList() {
        controls = new LinkedList<>();
        controls.add(buttonCreateCleaningCar);
        controls.add(buttonCreateTruck);

        controls.add(buttonUp);
        controls.add(buttonDown);
        controls.add(buttonLeft);
        controls.add(buttonRight);

        controls.add(comboBoxStrategy);
        controls.add(buttonStrategyStep);
    }
}